#field matcher for 104cm ST

from astropy.wcs import WCS
from astropy.io import fits
import numpy as np
import matplotlib.pyplot as plt
from astroquery.astrometry_net import AstrometryNet
from astropy import units as u
from astropy.coordinates import SkyCoord
import sys
from astroquery.simbad import Simbad
import warnings
warnings.filterwarnings("ignore")

def Field_Matcher(filename):
	ast=AstrometryNet()
	ast.api_key= 'iqmqwvazpvolmjmn'
	wcs_header=ast.solve_from_image(filename)
	wcs=WCS(wcs_header)
	ra=wcs.wcs.crval[0]
	dec=wcs.wcs.crval[1]
	c=SkyCoord(ra=ra*u.degree,dec=dec*u.degree)
	print("Center of image is ",c.to_string('hmsdms'))

	target_name=str(input("Enter the  name of target "))
	result_table = Simbad.query_object(target_name)

	ra_targte,dec_target=str(result_table['RA'].data[0]),str(result_table['DEC'].data[0])
	c_target=SkyCoord(ra=ra_targte,dec=dec_target,unit=(u.hourangle, u.deg))

	target_ra_h,target_ra_m,target_ra_s=c_target.ra.hms[0],c_target.ra.hms[1],c_target.ra.hms[2]
	ra_h,ra_m,ra_s=c.ra.hms[0],c.ra.hms[1],c.ra.hms[2]


	target_dec_h,target_dec_m,target_dec_s=c_target.dec.hms[0],c_target.dec.hms[1],c_target.dec.hms[2]
	dec_h,dec_m,dec_s=c.dec.hms[0],c.dec.hms[1],c.dec.hms[2]

	print("Your target will be at center if you move \nR.A. by ",target_ra_h-ra_h, "h",target_ra_m-ra_m, "m",target_ra_s-ra_s, "s\nDec by ",target_dec_h-dec_h,' d ',target_dec_m-dec_m, ' m ',target_dec_s-dec_s , ' s ')

	hdu=fits.open(sys.argv[1])[0]
	image=hdu.data
	header=hdu.header
	hdu.header.update(wcs.to_header())
	hdu.writeto('new.fits', output_verify='ignore',overwrite=True)




import numpy as np
import matplotlib as mp
import ccdproc,os,sys,time,random
from astropy import units as u
import matplotlib.pyplot as plt
from astropy.io import fits
from astropy.time import Time
from glob import glob
#from astroquery.astrometry_net import AstrometryNet
from astropy.wcs import WCS
import astroalign as aa
from astropy.coordinates import SkyCoord
#from pyraf import iraf
#from iraf import noao,imred,specred
from astropy.nddata import CCDData
from astropy.stats import sigma_clipped_stats, SigmaClip
from astropy.visualization import ImageNormalize, LogStretch
from matplotlib.ticker import LogLocator
from photutils.background import Background2D, MeanBackground,SExtractorBackground
from photutils import find_peaks, CircularAperture, CircularAnnulus, aperture_photometry
from photutils.centroids import centroid_2dg
from astropy.stats import SigmaClip, mad_std
from photutils import Background2D, MedianBackground, DAOStarFinder
from photutils.utils import calc_total_error
#from photutils.detection import findstars
from glob import glob
import sys
from astropy.time import Time
from photutils.morphology import data_properties	
from astropy.modeling.fitting import LevMarLSQFitter
from astropy.modeling.models import Const1D, Const2D, Gaussian1D, Gaussian2D




#dira = '.'
#gain = 2 * u.electron / u.adu  # float(input("Enter gain value")) gain and readout noise are properties of the CCD and will change for different CCDs.
#readnoise = 7.5 * u.electron    # float(input("Enter Read out noise value"))


def cleaning(dira,gain,readnoise):
	gain*=u.electron / u.adu
	readnoise*=u.electron
	mastebia=sorted(glob(os.path.join(dira,'masterbias.fits')))
	aa='n'
	if len(mastebia)==1:
		aa=str(input("Master BIAS already generated. want to regenerate?(y/n)"))

	if aa=='y' or len(mastebia)==0:
		bias_files = sorted(glob(os.path.join(dira,sys.argv[1])))
		biaslist = []
		for i in range (0,len(bias_files)):
			data= ccdproc.CCDData.read(bias_files[i],unit='adu')
			biaslist.append(data)

		masterbias = ccdproc.combine(biaslist,method='median',sigma_clip=True,sigma_clip_low_thresh=5,sigma_clip_high_thresh=5,sigma_clip_func=np.ma.median, sigma_clip_dev_func=mad_std)
		masterbias.write('masterbias.fits', overwrite=True)
		mbias=ccdproc.CCDData.read('masterbias.fits',unit='adu')
		print('Master bias generated')
		print(" Mean and median of the masterbias: ",np.mean(masterbias), np.median(masterbias))



	flat_files=sorted(glob(os.path.join(dira,sys.argv[2])))
	print(flat_files)
	def inv_median(a):
	    return 1 / np.median(a)


	counter=0
	mastefla=sorted(glob(os.path.join(dira,'masterflat.fits')))
	if len(mastefla)==1:
		aa=str(input("Master FLAT  has already been generated. want to regenerate?(y/n)"))

	if aa=='y' or len(mastefla)==0:
		print(counter,'masterflat.fits')
		flatlist = []
	for k in flat_files:
		flat=ccdproc.CCDData.read(k,unit='adu')
		flat_bias_removed=ccdproc.subtract_bias(flat,masterbias)
		flatlist.append(flat_bias_removed)
		masterflat = ccdproc.combine(flatlist,method='median', scale=inv_median,sigma_clip=True, sigma_clip_low_thresh=5, sigma_clip_high_thresh=5,sigma_clip_func=np.ma.median, sigma_clip_dev_func=mad_std)
		masterflat.write('masterflat.fits', overwrite=True)
		mflat=ccdproc.CCDData.read('masterflat.fits',unit='adu')
		print('Master flat generated')
		print(" Mean and median of the masterflat: ",np.mean(masterflat), np.median(masterflat))

	print("Master BIAS and Master Flat are generated")


	file_names = sorted(glob(os.path.join(dira,sys.argv[3])))

	counter=0
	for k in file_names:	
		fs=fits.open(k)
		#print(fs[0].header['FILTER1'],fs[0].header['OBJECT'])
		image=ccdproc.CCDData.read(k,unit='adu')
		header=fits.getheader(k)
		mflat=ccdproc.CCDData.read('masterflat.fits',unit='adu')
		mbais=ccdproc.CCDData.read('masterbias.fits',unit='adu')
		bias_subtracted = ccdproc.subtract_bias(image, mbais)
		flat_corrected = ccdproc.flat_correct(bias_subtracted, mflat)
		cr_cleaned =ccdproc.cosmicray_lacosmic(flat_corrected,readnoise=7.5,sigclip=5,satlevel=65535,niter=20,cleantype='meanmask',gain_apply=True)
		clean_file=k.replace('.fits','')
		fits.writeto(clean_file+'_cleaned.fits',cr_cleaned,header,output_verify='ignore',overwrite=True)
		print('Cleaning done on ',k )	



#cleaning()



import numpy as np
from astropy import units as u
import matplotlib.pyplot as plt
from astropy.io import fits
from astropy.time import Time
from glob import glob
from astroquery.astrometry_net import AstrometryNet
from astropy.wcs import WCS
from astropy.coordinates import SkyCoord
import sys
from astropy.time import Time


def astrometry_from_image(filename):
	f=open(list_of_filenames)
	lines=f.readlines()

	for line in lines:
		ast=AstrometryNet()
		ast.api_key= 'iqmqwvazpvolmjmn'
		wcs_header=ast.solve_from_image(line.split()[0])
		wcs=WCS(wcs_header)
		hdu=fits.open(line.split()[0])[0]
		image=hdu.data
		header=hdu.header
		hdu.header.update(wcs.to_header())
		hdu.writeto(str(line.split()[0]), output_verify='ignore',overwrite=True)

	f.close()


def astrometry_from_list(file_containing_list_of_files)
	f=open(file_containing_list_of_files)
	lines=f.readlines()

	for line in lines:
		ast=AstrometryNet()
		ast.api_key= 'iqmqwvazpvolmjmn'
		wcs_header=ast.solve_from_image(line.split()[0])
		wcs=WCS(wcs_header)
		hdu=fits.open(line.split()[0])[0]
		image=hdu.data
		header=hdu.header
		hdu.header.update(wcs.to_header())
		hdu.writeto(str(line.split()[0]), output_verify='ignore',overwrite=True)

	f.close()
